@@ -7,10 +7,10 @@ from rich.table import Table
 
 from .__version__ import __version__
 from .log import logger
-from .assign import assign
-from .handle import HandleMethodError
+from .handle import Handler
 from .progress import CLIProgress
-from .utils import parse_bytes_str
+from .utils import parse_bytes_str, s2t
+from .exception import HandleMethodError
 
 
 def handle_help(ctx: click.Context, param: typing.Union[click.Option, click.Parameter], value: typing.Any, ) -> None:
@@ -41,29 +41,29 @@ def print_help():
     console.print(f"\n[bold]bilix {__version__}", justify="center")
     console.print("⚡️快如闪电的bilibili下载工具，基于Python现代Async特性，高速批量下载整部动漫，电视剧，up投稿等\n",
                   justify="center")
-    console.print("使用方法： bilix [cyan]<method> <key> [OPTIONS][/cyan] ", justify="left")
+    console.print("使用方法： bilix [cyan]<method> <key1, key2...> [OPTIONS][/cyan] ", justify="left")
     table = Table.grid(padding=1, pad_edge=False)
     table.add_column("Parameter", no_wrap=True, justify="left", style="bold")
     table.add_column("Description")
 
     table.add_row(
         "[cyan]<method>",
-        'get_series 或 s：获取整个系列的视频（包括多p投稿，动漫，电视剧，电影，纪录片），也可以下载单个视频\n'
-        'get_video 或 v：获取特定的单个视频，在用户不希望下载系列其他视频的时候可以使用\n'
-        'get_up 或 up：获取某个up的所有投稿视频，支持数量选择，关键词搜索，排序\n'
-        'get_cate 或 cate：获取分区视频，支持数量选择，关键词搜索，排序\n'
-        'get_favour 或 fav：获取收藏夹内视频，支持数量选择，关键词搜索\n'
+        'get_series 或 s：   获取整个系列的视频（包括多p投稿，动漫，电视剧，电影，纪录片），也可以下载单个视频\n'
+        'get_video 或 v：    获取特定的单个视频，在用户不希望下载系列其他视频的时候可以使用\n'
+        'get_up 或 up：      获取某个up的所有投稿视频，支持数量选择，关键词搜索，排序\n'
+        'get_cate 或 cate：  获取分区视频，支持数量选择，关键词搜索，排序\n'
+        'get_favour 或 fav： 获取收藏夹内视频，支持数量选择，关键词搜索\n'
         'get_collect 或 col：获取合集或视频列表内视频\n'
-        'info：打印url所属资源的详细信息（例如点赞数，画质，编码格式等）'
+        'info：              打印url所属资源的详细信息（例如点赞数，画质，编码格式等）'
     )
     table.add_row(
-        "[cyan]<key>",
-        '如果使用get_video或get_series，在此填写视频的url\n'
-        '如果使用get_up，则在该位置填写b站用户空间页url或用户id\n'
-        '如果使用get_cate，则在该位置填写分区名称\n'
-        '如果使用get_favour，则在该位置填写收藏夹页url或收藏夹id\n'
-        '如果使用get_collect，则在该位置填写合集或者视频列表详情页url\n'
-        '如果使用info，则在该位置填写任意资源url'
+        "[cyan]<key>[/cyan]",
+        '如使用get_video/get_series，填写视频的url\n'
+        '如使用get_up，填写b站用户空间页url或用户id\n'
+        '如使用get_cate，填写分区名称\n'
+        '如使用get_favour，填写收藏夹页url或收藏夹id\n'
+        '如使用get_collect，填写合集或者视频列表详情页url\n'
+        '如使用info，填写任意资源url'
     )
     console.print(table)
     # console.rule("OPTIONS参数")
@@ -72,7 +72,7 @@ def print_help():
     table.add_column("type", no_wrap=True, justify="left", style="bold")
     table.add_column("Description", )
     table.add_row(
-        "--dir",
+        "-d --dir",
         '[dark_cyan]str',
         "文件的下载目录，默认当前路径下的videos文件夹下，不存在会自动创建"
     )
@@ -82,12 +82,12 @@ def print_help():
         "视频画面质量，默认0为最高画质，越大画质越低，超出范围时自动选最低画质，或者直接使用字符串指定'1080p'等名称"
     )
     table.add_row(
-        "--max-con",
+        "-mc --max-con",
         '[dark_cyan]int',
         "控制最大同时下载的视频数量，理论上网络带宽越高可以设的越高，默认3",
     )
     table.add_row(
-        "--part-con",
+        "-pc --part-con",
         '[dark_cyan]int',
         "控制每个媒体的分段并发数，默认10",
     )
@@ -96,6 +96,10 @@ def print_help():
         '[dark_cyan]str',
         '有条件的用户可以提供大会员的SESSDATA来下载会员视频'
     )
+    table.add_row(
+        "-fb --from-browser", '[dark_cyan]str',
+        '从哪个浏览器中导入cookies，例如safari，chrome，edge...默认无',
+    )
     table.add_row(
         '--days',
         '[dark_cyan]int',
@@ -117,11 +121,11 @@ def print_help():
         '搜索关键词， 仅get_up, get_cate，get_favor时生效',
     )
     table.add_row(
-        "--no-series", '',
+        "-ns --no-series", '',
         '只下载搜索结果每个视频的第一p，仅get_up，get_cate，get_favour时生效',
     )
     table.add_row(
-        "--no-hierarchy", '',
+        "-nh --no-hierarchy", '',
         '不使用层次目录，所有视频统一保存在下载目录下'
     )
     table.add_row(
@@ -137,7 +141,7 @@ def print_help():
         '下载弹幕',
     )
     table.add_row(
-        "--only-audio", '',
+        "-oa --only-audio", '',
         '仅下载音频，下载的音质固定为最高音质',
     )
     table.add_row(
@@ -150,7 +154,15 @@ def print_help():
     )
     table.add_row(
         "-sl --speed-limit", '[dark_cyan]str',
-        '最大下载速度，默认无限制。例如：-sl 1.5MB (experimental)',
+        '最大下载速度，默认无限制。例如：-sl 1.5MB',
+    )
+    table.add_row(
+        "-sr --stream-retry", '[dark_cyan]int',
+        '下载过程中发生网络错误后最大重试数，默认5',
+    )
+    table.add_row(
+        "-tr --time-range", '[dark_cyan]str',
+        r'下载视频的时间范围，格式如 h:m:s-h:m:s 或 s-s，默认无，仅get_video时生效',
     )
     table.add_row("-h --help", '', "帮助信息")
     table.add_row("-v --version", '', "版本信息")
@@ -180,10 +192,19 @@ class BasedSpeedLimit(click.ParamType):
             return parse_bytes_str(value)
 
 
+class BasedTimeRange(click.ParamType):
+    name = "time_range"
+
+    def convert(self, value, param, ctx):
+        start_time, end_time = map(s2t, value.split('-'))
+        return start_time, end_time
+
+
 @click.command(add_help_option=False)
 @click.argument("method", type=str)
-@click.argument("key", type=str)
+@click.argument("keys", type=str, nargs=-1, required=True)
 @click.option(
+    "-d",
     "--dir",
     "videos_dir",
     type=str,
@@ -197,12 +218,14 @@ class BasedSpeedLimit(click.ParamType):
     default=0,  # default relatively choice
 )
 @click.option(
+    '-mc',
     '--max-con',
     'video_concurrency',
     type=int,
     default=3,
 )
 @click.option(
+    '-pc',
     "--part-con",
     "part_concurrency",
     type=int,
@@ -237,12 +260,14 @@ class BasedSpeedLimit(click.ParamType):
     type=str
 )
 @click.option(
+    '-ns',
     '--no-series',
-    'no_series',
+    'series',
     is_flag=True,
     default=True,
 )
 @click.option(
+    '-nh',
     '--no-hierarchy',
     'hierarchy',
     is_flag=True,
@@ -267,6 +292,7 @@ class BasedSpeedLimit(click.ParamType):
     default=False,
 )
 @click.option(
+    '-oa',
     '--only-audio',
     'only_audio',
     is_flag=True,
@@ -290,6 +316,26 @@ class BasedSpeedLimit(click.ParamType):
     type=BasedSpeedLimit(),
     default=None,
 )
+@click.option(
+    '--stream-retry',
+    '-sr',
+    'stream_retry',
+    type=int,
+    default=5
+)
+@click.option(
+    '--from-browser',
+    '-fb',
+    'browser',
+    type=str,
+)
+@click.option(
+    '--time-range',
+    '-tr',
+    'time_range',
+    type=BasedTimeRange(),
+    default=None,
+)
 @click.option(
     '-h',
     "--help",
@@ -318,7 +364,9 @@ def main(**kwargs):
     asyncio.set_event_loop(loop)
     logger.debug(f'CLI KEY METHOD and OPTIONS: {kwargs}')
     try:
-        executor, cor = assign(**kwargs)
+        # CLIProgress.switch_theme(gs="cyan", bs="dark_cyan")
+        CLIProgress.start()  # start progress
+        executor, cor = Handler.assign(kwargs)
         loop.run_until_complete(cor)
     except HandleMethodError as e:  # method no match
         logger.error(e)
