@@ -1,16 +1,18 @@
-import asyncio
 import re
-from dataclasses import dataclass
+from pydantic import BaseModel
 import httpx
 from bs4 import BeautifulSoup
 from bilix.utils import legal_title, req_retry
+from ._decorator import api
 
 BASE_URL = "https://jable.tv"
-_dft_headers = {'user-agent': 'PostmanRuntime/7.29.0', "Referer": BASE_URL}
+dft_client_settings = {
+    'headers': {'user-agent': 'PostmanRuntime/7.29.0', "Referer": BASE_URL},
+    'http2': False
+}
 
 
-@dataclass
-class VideoInfo:
+class VideoInfo(BaseModel):
     url: str
     avid: str
     title: str
@@ -19,6 +21,7 @@ class VideoInfo:
     img_url: str
 
 
+@api
 async def get_model_info(client: httpx.AsyncClient, url: str):
     res = await req_retry(client, url)
     soup = BeautifulSoup(res.text, "html.parser")
@@ -27,6 +30,7 @@ async def get_model_info(client: httpx.AsyncClient, url: str):
     return {'model_name': model_name, 'urls': urls}
 
 
+@api
 async def get_video_info(client: httpx.AsyncClient, url_or_avid: str) -> VideoInfo:
     if url_or_avid.startswith('http'):
         url = url_or_avid
