@@ -1,24 +1,26 @@
-import asyncio
 import re
-from dataclasses import dataclass
-from typing import Union, Sequence
+from pydantic import BaseModel
+from typing import Union, List
 import httpx
 from bs4 import BeautifulSoup
-from bilix.log import logger
 from bilix.utils import req_retry
+from ._decorator import api
 
 BASE_URL = "http://www.yinghuacd.com"
-_dft_headers = {'user-agent': 'PostmanRuntime/7.29.0'}
+dft_client_settings = {
+    'headers': {'user-agent': 'PostmanRuntime/7.29.0'},
+    'http2': False
+}
 
 
-@dataclass
-class VideoInfo:
+class VideoInfo(BaseModel):
     title: str
     sub_title: str
-    play_info: Sequence[Union[Sequence[str], Sequence]]  # may be empty
+    play_info: List[Union[List[str], List]]  # may be empty
     m3u8_url: str
 
 
+@api
 async def get_video_info(client: httpx.AsyncClient, url: str) -> VideoInfo:
     # request
     res = await req_retry(client, url)
@@ -31,17 +33,3 @@ async def get_video_info(client: httpx.AsyncClient, url: str) -> VideoInfo:
     play_info = [[a.text, f"{BASE_URL}{a['href']}"] for a in soup.find('div', class_="movurls").find_all('a')]
     video_info = VideoInfo(title=title, sub_title=sub_title, play_info=play_info, m3u8_url=m3u8_url)
     return video_info
-
-
-async def main():
-    _dft_client = httpx.AsyncClient(headers=_dft_headers, http2=True)
-
-    return await asyncio.gather(
-        get_video_info(_dft_client, "http://www.yinghuacd.com/v/5606-7.html"),
-    )
-
-
-if __name__ == '__main__':
-    logger.setLevel("DEBUG")
-    result = asyncio.run(main())
-    print(result)
