@@ -1,39 +1,41 @@
 import asyncio
+import platform
 from typing import Union, Optional
 from contextlib import asynccontextmanager
 import aiofiles
 import httpx
 import os
-from bilix.utils import req_retry
+from bilix.utils import req_retry, update_cookies_from_browser
 from bilix.log import logger
-from bilix.progress import CLIProgress, BaseProgress
+from bilix.progress.abc import Progress
+from bilix.progress import CLIProgress
 
 
 class BaseDownloader:
-    LIMIT_BOUND: float = 1e5
-    DELAY_SLOPE: float = 0.1
-
-    def __init__(self, client: httpx.AsyncClient, videos_dir='videos', speed_limit: Union[float, int] = None,
-                 progress: BaseProgress = None):
+    def __init__(self, client: httpx.AsyncClient = None, videos_dir='videos',
+                 video_concurrency: Union[int, asyncio.Semaphore] = 3, part_concurrency: int = 10,
+                 speed_limit: Union[float, int] = None, stream_retry=5, progress: Progress = None, browser: str = None):
         """
 
         :param client: client used for http request
+        :param browser: load cookies from which browser
         :param videos_dir: download to which directory, default to ./videos, if not exists will be auto created
         :param speed_limit: global download rate for the downloader, should be a number (Byte/s unit)
         :param progress: progress obj
         """
-        self.client = client
+        self.client = client if client else httpx.AsyncClient(headers={'user-agent': 'PostmanRuntime/7.29.0'})
+        if browser:  # load cookies from browser, may need auth
+            update_cookies_from_browser(self.client, browser, self.domain if hasattr(self, "domain") else "")
         self.videos_dir = videos_dir
         assert speed_limit is None or speed_limit > 0
         self.speed_limit = speed_limit
         if not os.path.exists(self.videos_dir):
             os.makedirs(videos_dir)
-        if progress is None:
-            # if no progress_cls provided by upper class, use cli progress by default
-            self.progress = CLIProgress(holder=self)
-        else:
-            self.progress = progress
-            progress.holder = self
+        # if no progress_cls provided by upper class, use cli progress by default
+        self.progress = progress or CLIProgress()
+        self.stream_retry = stream_retry
+        self.v_sema = asyncio.Semaphore(video_concurrency) if type(video_concurrency) is int else video_concurrency
+        self.part_concurrency = part_concurrency
         # active stream number
         self._stream_num = 0
 
@@ -56,6 +58,8 @@ class BaseDownloader:
         :return:
         """
         assert hierarchy is True or (type(hierarchy) is str and len(hierarchy) > 0) and len(add_dir) > 0
+        if platform.system() == 'Windows':
+            add_dir = add_dir.rstrip('.')
         hierarchy = add_dir if hierarchy is True else f'{hierarchy}/{add_dir}'
         if not os.path.exists(f'{self.videos_dir}/{hierarchy}'):
             os.makedirs(f'{self.videos_dir}/{hierarchy}')
@@ -122,10 +126,20 @@ class BaseDownloader:
         """current activate network stream number"""
         return self._stream_num
 
+    LIMIT_BOUND: float = 1e5
+    DELAY_SLOPE: float = 0.1
+
     @property
     def chunk_size(self) -> Optional[int]:
-        if self.speed_limit and self.progress and self.speed_limit < self.LIMIT_BOUND:
+        if self.speed_limit and self.speed_limit < self.LIMIT_BOUND:
             # only restrict chunk_size when speed_limit is too low
             return int(self.speed_limit * self.DELAY_SLOPE)
         # default to None setup
         return None
+
+    async def _check_speed(self, content_size):
+        if self.speed_limit and (cur_speed := self.progress.active_speed) > self.speed_limit:
+            t_tgt = content_size / self.speed_limit * self.stream_num
+            t_real = content_size / cur_speed
+            t = t_tgt - t_real
+            await asyncio.sleep(t)
