@@ -1,91 +1,52 @@
 import asyncio
-from typing import Union, Sequence, Tuple, Optional
+import functools
+from typing import Union, Sequence, Tuple, List
 import aiofiles
 import httpx
 from datetime import datetime, timedelta
 import os
-from itertools import groupby
 from anyio import run_process
 import bilix.api.bilibili as api
-from bilix.handle import Handler, HandleMethodError
+from bilix.handle import Handler
 from bilix.download.base_downloader_part import BaseDownloaderPart
+from bilix.process import SingletonPPE
 from bilix.subtitle import json2srt
-from bilix.dm import dm2ass_factory
-from bilix.utils import legal_title, req_retry, cors_slice, parse_bilibili_url
+from bilix.utils import legal_title, req_retry, cors_slice, parse_bilibili_url, valid_sess_data, t2s
 from bilix.log import logger
-
-__all__ = ['DownloaderBilibili']
-
-
-def choose_quality(video_info: api.VideoInfo, quality: Union[str, int], codec: str = '') -> Tuple[dict, dict]:
-    """
-
-    :param video_info:
-    :param quality:
-    :param codec: should be like xxx (for video only) :xxxx (for audio only) xxx:xxx (for video and audio)
-    :return: video, audio, video_urls, audio_urls
-    """
-    dash, support_formats = video_info.dash, video_info.support_formats
-    v_codec, a_codec, *_ = codec.split(':') + [""]
-    # for audio
-    if a_codec == '':
-        audio = dash['audio'][0]
-        audio['suffix'] = '.aac'
-    else:
-        if dash['dolby']['audio'] and dash['dolby']['audio'][0]['codecs'].startswith(a_codec):
-            audio = dash['dolby']['audio'][0]
-            audio['suffix'] = '.eac3'  # todo other dolby codec?
-        elif dash.get('flac', None) and dash['flac']['audio'] and dash['flac']['audio']['codecs'].startswith(a_codec):
-            audio = dash['flac']['audio']
-            audio['suffix'] = '.flac'
-        else:
-            raise ValueError(f'Invalid quality and codec quality:{quality} codec: {codec}')
-    audio['urls'] = (audio['base_url'], *(audio['backup_url'] if audio['backup_url'] else ()))
-
-    # for video
-    if isinstance(quality, str):  # 1. absolute choice with quality name like 4k 1080p '1080p 60帧'
-        for f_info in support_formats:
-            if f_info['new_description'].upper().startswith(quality.upper()):
-                q_id = f_info['quality']
-                for video in dash['video']:
-                    if video['id'] == q_id and (v_codec == '' or video['codecs'].startswith(v_codec)):
-                        video['urls'] = (video['base_url'], *(video['backup_url'] if video['backup_url'] else ()))
-                        logger.debug(f"'{video_info.title}' quality <{f_info['new_description']}>"
-                                     f" codec <{video['codecs']}:{audio['codecs']}> has been chosen")
-                        return video, audio
-    else:  # 2. relative choice
-        quality = min(quality, len(set(i['id'] for i in dash['video'])) - 1)
-        for q, (q_id, it) in enumerate(groupby(dash['video'], key=lambda x: x['id'])):
-            if q == quality:
-                for video in it:
-                    if v_codec == '' or video['codecs'].startswith(v_codec):
-                        video['urls'] = (video['base_url'], *(video['backup_url'] if video['backup_url'] else ()))
-                        logger.debug(f"'{video_info.title}' relative quality <{quality}>"
-                                     f" codec <{video['codecs']}:{audio['codecs']}> has been chosen")
-                        return video, audio
-    raise ValueError(f'Invalid quality and codec quality:{quality} codec: {codec}')
+from bilix.exception import HandleMethodError, APIUnsupportedError, APIResourceError, APIError
+from danmakuC.bilibili import proto2ass
 
 
 class DownloaderBilibili(BaseDownloaderPart):
-    def __init__(self, videos_dir='videos', sess_data='', video_concurrency=3, part_concurrency=10,
-                 speed_limit: Union[float, int] = None, progress=None):
+    def __init__(self, videos_dir='videos', sess_data: str = None,
+                 video_concurrency: Union[int, asyncio.Semaphore] = 3, part_concurrency: int = 10,
+                 stream_retry=5, speed_limit: Union[float, int] = None, progress=None, browser=None):
         """
 
         :param videos_dir: 下载到哪个目录，默认当前目录下的为videos中，如果路径不存在将自动创建
-        :param sess_data: 有条件的用户填写大会员凭证，填写后可下载大会员资源
+        :param sess_data: 有条件的用户填写大会员凭证，填3写后可下载大会员资源
         :param video_concurrency: 限制最大同时下载的视频数量
         :param part_concurrency: 每个媒体的分段并发数
+        :param stream_retry: 下载媒体时发生网络错误的最大重试数
         :param speed_limit: 下载速度限制，单位B/s
         :param progress: 进度对象，不提供则使用rich命令行进度
+        :param browser: 从哪个浏览器中读取cookies，例如chrome，safari
         """
-        cookies = {'SESSDATA': sess_data}
-        headers = {'user-agent': 'PostmanRuntime/7.29.0', 'referer': 'https://www.bilibili.com'}
-        client = httpx.AsyncClient(headers=headers, cookies=cookies, http2=True)
-        super(DownloaderBilibili, self).__init__(client, videos_dir, part_concurrency, speed_limit=speed_limit,
-                                                 progress=progress)
-        self.speed_limit = speed_limit
-        self.v_sema = asyncio.Semaphore(video_concurrency)
+        self.domain = "bilibili.com"
+        client = httpx.AsyncClient(**api.dft_client_settings)
+        client.cookies.set('SESSDATA', valid_sess_data(sess_data))
+        super(DownloaderBilibili, self).__init__(
+            client=client,
+            browser=browser,
+            videos_dir=videos_dir,
+            part_concurrency=part_concurrency,
+            video_concurrency=video_concurrency,
+            speed_limit=speed_limit,
+            stream_retry=stream_retry,
+            progress=progress
+        )
         self._cate_meta = None
+        self.api_sema = asyncio.Semaphore(video_concurrency)
 
     async def get_collect_or_list(self, url, quality=0, image=False, subtitle=False, dm=False, only_audio=False,
                                   codec: str = '', hierarchy: Union[bool, str] = True):
@@ -104,57 +65,14 @@ class DownloaderBilibili(BaseDownloaderPart):
         """
         t = parse_bilibili_url(url)
         if t == 'list':
-            cor = self.get_list
+            list_name, up_name, bvids = await api.get_list_info(self.client, url)
+            name = legal_title(f"【视频列表】{up_name}", list_name)
         elif t == 'col':
-            cor = self.get_collect
+            col_name, up_name, bvids = await api.get_collect_info(self.client, url)
+            name = legal_title(f"【合集】{up_name}", col_name)
         else:
             raise ValueError(f'{url} invalid for get_collect_or_list')
-        # noinspection PyArgumentList
-        await cor(url=url, quality=quality, image=image, subtitle=subtitle, dm=dm, only_audio=only_audio,
-                  codec=codec, hierarchy=hierarchy)
-
-    async def get_list(self, url, quality=0, image=False, subtitle=False, dm=False, only_audio=False,
-                       codec: str = '', hierarchy: Union[bool, str] = True):
-        """
-        下载视频列表
-
-        :param url: 列表详情页url
-        :param quality:
-        :param image:
-        :param subtitle:
-        :param dm:
-        :param only_audio:
-        :param codec
-        :param hierarchy:
-        :return:
-        """
-        list_name, up_name, bvids = await api.get_list_info(self.client, url)
         if hierarchy:
-            name = legal_title(f"【视频列表】{up_name}-{list_name}")
-            hierarchy = self._make_hierarchy_dir(hierarchy, name)
-        await asyncio.gather(
-            *[self.get_series(f"https://www.bilibili.com/video/{i}", quality=quality, codec=codec,
-                              image=image, subtitle=subtitle, dm=dm, only_audio=only_audio, hierarchy=hierarchy)
-              for i in bvids])
-
-    async def get_collect(self, url, quality=0, image=False, subtitle=False, dm=False, only_audio=False,
-                          codec: str = '', hierarchy: Union[bool, str] = True):
-        """
-        下载合集
-
-        :param url: 合集详情页url
-        :param quality: 画面质量，0为可以观看的最高画质，越大质量越低，超过范围时自动选择最低画质，或者直接使用字符串指定'1080p'等名称
-        :param image: 是否下载封面
-        :param subtitle: 是否下载字幕
-        :param dm: 是否下载弹幕
-        :param only_audio: 是否仅下载音频
-        :param codec:
-        :param hierarchy:
-        :return:
-        """
-        col_name, up_name, bvids = await api.get_collect_info(self.client, url)
-        if hierarchy:
-            name = legal_title(f"【合集】{up_name}-{col_name}")
             hierarchy = self._make_hierarchy_dir(hierarchy, name)
         await asyncio.gather(
             *[self.get_series(f"https://www.bilibili.com/video/{i}", quality=quality, codec=codec,
@@ -241,12 +159,10 @@ class DownloaderBilibili(BaseDownloaderPart):
         """
         cate_meta = await self.cate_meta
         if cate_name not in cate_meta:
-            logger.error(f'未找到分区 {cate_name}')
-            return
+            return logger.error(f'未找到分区 {cate_name}')
         if 'subChannelId' not in cate_meta[cate_name]:
             sub_names = [i['name'] for i in cate_meta[cate_name]['sub']]
-            logger.error(f'{cate_name} 是主分区，仅支持子分区，试试 {sub_names}')
-            return
+            return logger.error(f'{cate_name} 是主分区，仅支持子分区，试试 {sub_names}')
         if hierarchy:
             hierarchy = self._make_hierarchy_dir(hierarchy, legal_title(f"【分区】{cate_name}"))
         cate_id = cate_meta[cate_name]['tid']
@@ -344,34 +260,29 @@ class DownloaderBilibili(BaseDownloaderPart):
         :return:
         """
         try:
-            video_info = await api.get_video_info(self.client, url)
-        except AttributeError as e:
-            logger.warning(f'{e} {url}')
-            return
-        title = video_info.title
-        pages = video_info.pages
-        p = video_info.p
-        if hierarchy and len(pages) > 1:
-            hierarchy = self._make_hierarchy_dir(hierarchy, title)
+            async with self.api_sema:
+                video_info = await api.get_video_info(self.client, url)
+        except (APIResourceError, APIUnsupportedError) as e:
+            return logger.warning(e)
+        if hierarchy and len(video_info.pages) > 1:
+            hierarchy = self._make_hierarchy_dir(hierarchy, video_info.title)
         else:
             hierarchy = hierarchy if type(hierarchy) is str else ''  # incase hierarchy is False
-        cors = [self.get_video(p_url, quality, add_name,
-                               image=image,
-                               subtitle=subtitle, dm=dm, only_audio=only_audio, codec=codec, hierarchy=hierarchy,
-                               video_info=video_info if idx == p else None)
-                for idx, (add_name, p_url) in enumerate(pages)]
+        cors = [self.get_video(p.p_url, quality=quality, image=image, subtitle=subtitle, dm=dm, only_audio=only_audio,
+                               codec=codec, hierarchy=hierarchy, video_info=video_info if idx == video_info.p else None)
+                for idx, p in enumerate(video_info.pages)]
         if p_range:
             cors = cors_slice(cors, p_range)
         await asyncio.gather(*cors)
 
-    async def get_video(self, url: str, quality: Union[str, int] = 0, add_name='', image=False, subtitle=False,
-                        dm=False, only_audio=False, codec: str = '', hierarchy: str = '', video_info=None):
+    async def get_video(self, url: str, quality: Union[str, int] = 0, image=False, subtitle=False,
+                        dm=False, only_audio=False, codec: str = '', hierarchy: str = '',
+                        video_info: api.VideoInfo = None, time_range: Tuple[int, int] = None):
         """
         下载单个视频
 
         :param url: 视频的url
         :param quality: 画面质量，0为可以观看的最高画质，越大质量越低，超过范围时自动选择最低画质，或者直接使用字符串指定'1080p'等名称
-        :param add_name: 给文件的额外添加名，用户请直接保持默认
         :param image: 是否下载封面
         :param subtitle: 是否下载字幕
         :param dm: 是否下载弹幕
@@ -379,71 +290,113 @@ class DownloaderBilibili(BaseDownloaderPart):
         :param codec: 视频编码（可通过codec获取）
         :param hierarchy:
         :param video_info: 额外数据，提供时不用再次请求页面
+        :param time_range: 切片的时间范围
         :return:
         """
         async with self.v_sema:
             if not video_info:
                 try:
                     video_info = await api.get_video_info(self.client, url)
-                except AttributeError as e:
-                    logger.warning(f'{url} {e}')
-                    return
-            title = video_info.h1_title
-            title = legal_title(title, add_name)
-            video_info.title = title  # update video_info title
-            img_url = video_info.img_url
-            if not video_info.dash:
-                logger.warning(f'{video_info.title} 需要大会员或该地区不支持')
-                return
-            # choose video quality
-            try:
-                video, audio = choose_quality(video_info, quality, codec)
-            except ValueError:
-                logger.warning(
-                    f"{video_info.title} 清晰度<{quality}> 编码<{codec}>不可用，请检查输入是否正确或是否需要大会员")
-                return
-
+                except (APIResourceError, APIUnsupportedError) as e:
+                    return logger.warning(e)
+            # join p_name and title
+            p_name = video_info.pages[video_info.p].p_name
+            if time_range:
+                p_name = legal_title(p_name, *map(t2s, time_range))
+            title = legal_title(video_info.h1_title, p_name)
+            # to avoid file name too long bug
+            file_name = p_name if len(video_info.h1_title) > 50 and hierarchy and p_name else title
             file_dir = f'{self.videos_dir}/{hierarchy}' if hierarchy else self.videos_dir
-            task_id = await self.progress.add_task(
-                total=1, description=title if len(title) < 33 else f'{title[:15]}...{title[-15:]}', visible=False)
+
             cors = []
-            # add cor according to params
-            if not only_audio:
-                if os.path.exists(f'{file_dir}/{title}.mp4'):
-                    logger.info(f'[green]已存在[/green] {title}.mp4')
+            task_id = await self.progress.add_task(total=None, description=title)
+            if video_info.dash:
+                try:  # choose video quality
+                    video, audio = video_info.dash.choose_quality(quality, codec)
+                except KeyError:
+                    return logger.warning(
+                        f"{title} 清晰度<{quality}> 编码<{codec}>不可用，请检查输入是否正确或是否需要大会员")
+
+                tmp: List[Tuple[api.Media, str]] = []
+                # 1. only video
+                if not audio and not only_audio:
+                    tmp.append((video, f'{file_name}.mp4'))
+                # 2. video and audio
+                elif audio and not only_audio:
+                    if os.path.exists(f'{file_dir}/{file_name}.mp4'):
+                        logger.info(f'[green]已存在[/green] {file_name}.mp4')
+                    else:
+                        tmp.append((video, f'{file_name}-v'))
+                        tmp.append((audio, f'{file_name}-a'))
+                        # task need to be merged
+                        await self.progress.update(task_id=task_id, upper=True)
+                # 3. only audio
+                elif audio and only_audio:
+                    tmp.append((audio, f'{file_name}{audio.suffix}'))
                 else:
-                    cors.append(self.get_media(video['urls'], f'{title}-video', task_id, hierarchy))
-                    cors.append(self.get_media(audio['urls'], f'{title}-audio', task_id, hierarchy))
+                    return logger.warning(f"No audio for {file_name}")
+                # convert to coroutines
+                for t in tmp:
+                    if not time_range:
+                        cors.append(self.get_file(t[0].urls, t[1], task_id=task_id, hierarchy=hierarchy))
+                    else:
+                        cors.append(self.get_media_clip(
+                            t[0].urls, t[1],
+                            time_range=time_range,
+                            init_range=t[0].segment_base['initialization'],
+                            seg_range=t[0].segment_base['index_range'],
+                            task_id=task_id, hierarchy=hierarchy))
+            elif video_info.other:
+                logger.warning(
+                    f"{title} 未解析到dash资源，转入durl mp4/flv下载（不需要会员的电影/番剧预览，不支持dash的视频）")
+                if len(video_info.other) == 1:
+                    m = video_info.other[0]
+                    cors.append(self.get_file(m.urls, f'{file_name}.{m.suffix}', task_id, hierarchy=hierarchy))
+                else:
+                    # todo 暂时还没遇到这种情况
+                    sub_task = []
+                    for i, m in enumerate(video_info.other):
+                        sub_task.append(
+                            self.get_file(m.urls, f'{file_name}-{i}.{m.suffix}', task_id, hierarchy=hierarchy))
             else:
-                cors.append(self.get_media(audio['urls'], f'{title}{audio["suffix"]}', task_id, hierarchy))
+                return logger.warning(f'{title} 需要大会员或该地区不支持')
+
             # additional task
             if image or subtitle or dm:
                 extra_hierarchy = self._make_hierarchy_dir(hierarchy if hierarchy else True, 'extra')
                 if image:
-                    cors.append(self._get_static(img_url, title, hierarchy=extra_hierarchy))
+                    cors.append(self._get_static(video_info.img_url, file_name, hierarchy=extra_hierarchy))
                 if subtitle:
                     cors.append(self.get_subtitle(url, hierarchy=extra_hierarchy, video_info=video_info))
                 if dm:
-                    w, h = video['width'], video['height']
-                    cors.append(
-                        self.get_dm(url, convert_func=dm2ass_factory(w, h), hierarchy=extra_hierarchy,
-                                    video_info=video_info))
+                    width, height = (video.width, video.height) if video_info.dash else (1920, 1080)
+                    cors.append(self.get_dm(url, convert_func=self._dm2ass_factory(width, height),
+                                            hierarchy=extra_hierarchy, video_info=video_info))
             await asyncio.gather(*cors)
 
-        if not only_audio and not os.path.exists(f'{file_dir}/{title}.mp4'):
-            cmd = ['ffmpeg', '-i', f'{file_dir}/{title}-video', '-i', f'{file_dir}/{title}-audio',
+        if self.progress.tasks[task_id].fields.get('upper', None):
+            cmd = ['ffmpeg', '-i', f'{file_dir}/{file_name}-v', '-i', f'{file_dir}/{file_name}-a',
                    '-codec', 'copy', '-loglevel', 'quiet']
             # ffmpeg: flac in MP4 support is experimental, add '-strict -2' if you want to use it.
-            if audio['codecs'] == 'fLaC':
+            if audio.codec == 'fLaC':
                 cmd.extend(['-strict', '-2'])
-            cmd.append(f'{file_dir}/{title}.mp4')
+            cmd.append(f'{file_dir}/{file_name}.mp4')
             await run_process(cmd)
-            os.remove(f'{file_dir}/{title}-video')
-            os.remove(f'{file_dir}/{title}-audio')
+            os.remove(f'{file_dir}/{file_name}-v')
+            os.remove(f'{file_dir}/{file_name}-a')
+            logger.info(f'[cyan]已完成[/cyan] {file_name}{audio.suffix if only_audio else ".mp4"}')
         # make progress invisible
-        if self.progress.tasks[task_id].visible:
-            await self.progress.update(task_id, advance=1, visible=False)
-            logger.info(f'[cyan]已完成[/cyan] {title}{audio["suffix"] if only_audio else ".mp4"}')
+        await self.progress.update(task_id, visible=False)
+
+    @staticmethod
+    def _dm2ass_factory(width: int, height: int):
+        async def dm2ass(protobuf_bytes: bytes) -> bytes:
+            loop = asyncio.get_event_loop()
+            f = functools.partial(proto2ass, protobuf_bytes, width, height, font_size=width / 40, )
+            content = await loop.run_in_executor(SingletonPPE(), f)
+            return content.encode('utf-8')
+
+        return dm2ass
 
     async def get_dm(self, url, update=False, convert_func=None, hierarchy: str = '', video_info=None):
         """
@@ -457,17 +410,19 @@ class DownloaderBilibili(BaseDownloaderPart):
         """
         if not video_info:
             video_info = await api.get_video_info(self.client, url)
-        title = video_info.title
         aid, cid = video_info.aid, video_info.cid
-
         file_dir = f'{self.videos_dir}/{hierarchy}' if hierarchy else self.videos_dir
-        file_type = '.' + ('bin' if not convert_func else convert_func.__name__.split('2')[-1])
-        file_name = f"{title}-弹幕{file_type}"
+        file_type = '.' + ('pb' if not convert_func else convert_func.__name__.split('2')[-1])
+        p_name = video_info.pages[video_info.p].p_name
+        if len(video_info.h1_title) > 50 and hierarchy and p_name:  # to avoid file name too long bug
+            file_name = legal_title(p_name, "弹幕") + file_type
+        else:
+            file_name = legal_title(video_info.h1_title, p_name, "弹幕") + file_type
         file_path = f'{file_dir}/{file_name}'
         if not update and os.path.exists(file_path):
             logger.info(f"[green]已存在[/green] {file_name}")
             return file_path
-        dm_urls = await api.get_dm_info(self.client, aid, cid)
+        dm_urls = await api.get_dm_urls(self.client, aid, cid)
         cors = [req_retry(self.client, dm_url) for dm_url in dm_urls]
         results = await asyncio.gather(*cors)
         content = b''.join(res.content for res in results)
@@ -479,12 +434,12 @@ class DownloaderBilibili(BaseDownloaderPart):
         logger.info(f"[cyan]已完成[/cyan] {file_name}")
         return file_path
 
-    async def get_subtitle(self, url, convert=True, hierarchy: str = '', video_info=None):
+    async def get_subtitle(self, url, convert_func=json2srt, hierarchy: str = '', video_info=None):
         """
         获取某个视频的字幕文件
 
         :param url: 视频url
-        :param convert: 是否转换成srt
+        :param convert_func: function used to convert original subtitle text
         :param hierarchy:
         :param video_info: 额外数据，提供则不再访问前端
         :return:
@@ -492,73 +447,40 @@ class DownloaderBilibili(BaseDownloaderPart):
         if not video_info:
             video_info = await api.get_video_info(self.client, url)
         p, cid = video_info.p, video_info.cid
-        title = video_info.title
-        add_name = video_info.pages[p][0]
-        title = legal_title(title, add_name)
+        p_name = video_info.pages[p].p_name
         try:
             subtitles = await api.get_subtitle_info(self.client, video_info.bvid, cid)
-        except AttributeError as e:
-            logger.warning(f'{url} {e}')
-            return
+        except APIError as e:
+            return logger.warning(e)
         cors = []
+
         for sub_url, sub_name in subtitles:
-            file_name = f"{title}-{sub_name}"
-            cors.append(self._get_static(sub_url, file_name, convert_func=json2srt if convert else None,
-                                         hierarchy=hierarchy))
+            if len(video_info.h1_title) > 50 and hierarchy and p_name:  # to avoid file name too long bug
+                file_name = legal_title(p_name, sub_name)
+            else:
+                file_name = legal_title(video_info.h1_title, p_name, sub_name)
+            cors.append(self._get_static(sub_url, file_name, convert_func=convert_func, hierarchy=hierarchy))
         paths = await asyncio.gather(*cors)
         return paths
 
 
-@Handler(name='bilibili')
-def handle(
-        method: str,
-        key: str,
-        videos_dir: str,
-        video_concurrency: int,
-        part_concurrency: int,
-        cookie: str,
-        quality: Union[str, int],
-        days: int,
-        num: int,
-        order: str,
-        keyword: str,
-        no_series: bool,
-        hierarchy: bool,
-        image: bool,
-        subtitle: bool,
-        dm: bool,
-        only_audio: bool,
-        p_range,
-        codec: str,
-        speed_limit: Optional[str]
-):
-    d = DownloaderBilibili(videos_dir=videos_dir,
-                           video_concurrency=video_concurrency,
-                           part_concurrency=part_concurrency,
-                           speed_limit=speed_limit,
-                           sess_data=cookie)
+@Handler.register(name='bilibili')
+def handle(cli_kwargs):
+    d = DownloaderBilibili(sess_data=cli_kwargs['cookie'],
+                           **Handler.kwargs_filter(DownloaderBilibili, cli_kwargs=cli_kwargs))
+    method = cli_kwargs['method']
     if method == 'get_series' or method == 's':
-        cor = d.get_series(key, quality=quality, image=image, subtitle=subtitle, dm=dm, only_audio=only_audio,
-                           p_range=p_range, hierarchy=hierarchy, codec=codec)
+        m = d.get_series
     elif method == 'get_video' or method == 'v':
-        cor = d.get_video(key, quality=quality,
-                          image=image, subtitle=subtitle, dm=dm, only_audio=only_audio, codec=codec)
+        m = d.get_video
     elif method == 'get_up' or method == 'up':
-        cor = d.get_up_videos(
-            key, quality=quality, num=num, order=order, keyword=keyword, series=no_series,
-            image=image, subtitle=subtitle, dm=dm, only_audio=only_audio, hierarchy=hierarchy, codec=codec
-        )
+        m = d.get_up_videos
     elif method == 'get_cate' or method == 'cate':
-        cor = d.get_cate_videos(
-            key, quality=quality, num=num, order=order, keyword=keyword, days=days, series=no_series,
-            image=image, subtitle=subtitle, dm=dm, only_audio=only_audio, hierarchy=hierarchy, codec=codec)
+        m = d.get_cate_videos
     elif method == 'get_favour' or method == 'fav':
-        cor = d.get_favour(key, quality=quality, num=num, keyword=keyword, series=no_series, codec=codec,
-                           image=image, subtitle=subtitle, dm=dm, only_audio=only_audio, hierarchy=hierarchy)
+        m = d.get_favour
     elif method == 'get_collect' or method == 'col':
-        cor = d.get_collect_or_list(key, quality=quality, codec=codec,
-                                    image=image, subtitle=subtitle, dm=dm, only_audio=only_audio,
-                                    hierarchy=hierarchy)
+        m = d.get_collect_or_list
     else:
-        raise HandleMethodError(executor=d, method=method)
-    return d, cor
+        raise HandleMethodError(DownloaderBilibili, method=method)
+    return d, m
