@@ -1,14 +1,15 @@
 import httpx
 from rich.console import Console
-
-__all__ = ['BaseInformer']
+from bilix.utils import update_cookies_from_browser
 
 
 class BaseInformer:
     console = Console()
 
-    def __init__(self, client: httpx.AsyncClient):
+    def __init__(self, client: httpx.AsyncClient, browser: str = None):
         self.client = client
+        if browser:  # load cookies from browser, may need auth
+            update_cookies_from_browser(self.client, browser, self.domain if hasattr(self, "domain") else "")
 
     async def __aenter__(self):
         await self.client.__aenter__()
